@@ -1,28 +1,37 @@
 import os
-localDir = os.path.dirname(__file__)
 import threading
 import time
+import socket
+
+import pytest
 
 import cherrypy
-from cherrypy._cpcompat import copykeys, HTTPConnection, HTTPSConnection
+from cherrypy._cpcompat import (
+    copykeys, json_decode,
+    HTTPConnection, HTTPSConnection
+)
 from cherrypy.lib import sessions
+from cherrypy.lib import reprconf
 from cherrypy.lib.httputil import response_codes
+from cherrypy.test import helper
+
+localDir = os.path.dirname(__file__)
 
 
 def http_methods_allowed(methods=['GET', 'HEAD']):
     method = cherrypy.request.method.upper()
     if method not in methods:
-        cherrypy.response.headers['Allow'] = ", ".join(methods)
+        cherrypy.response.headers['Allow'] = ', '.join(methods)
         raise cherrypy.HTTPError(405)
 
-cherrypy.tools.allow = cherrypy.Tool('on_start_resource', http_methods_allowed)
+cherrypy.tools.allow = cherrypy.Tool('on_start_resource', http_methods_allowed)  # noqa: E305
 
 
 def setup_server():
 
     @cherrypy.config(**{
         'tools.sessions.on': True,
-        'tools.sessions.storage_type': 'ram',
+        'tools.sessions.storage_class': sessions.RamSession,
         'tools.sessions.storage_path': localDir,
         'tools.sessions.timeout': (1.0 / 60),
         'tools.sessions.clean_freq': (1.0 / 60),
@@ -52,16 +61,16 @@ def setup_server():
 
         @cherrypy.expose
         @cherrypy.config(**{'tools.sessions.on': False})
-        def setsessiontype(self, newtype):
-            self.__class__._cp_config.update(
-                {'tools.sessions.storage_type': newtype})
-            if hasattr(cherrypy, "session"):
+        def set_session_cls(self, new_cls_name):
+            new_cls = reprconf.attributes(new_cls_name)
+            cfg = {'tools.sessions.storage_class': new_cls}
+            self.__class__._cp_config.update(cfg)
+            if hasattr(cherrypy, 'session'):
                 del cherrypy.session
-            cls = getattr(sessions, newtype.title() + 'Session')
-            if cls.clean_thread:
-                cls.clean_thread.stop()
-                cls.clean_thread.unsubscribe()
-                del cls.clean_thread
+            if new_cls.clean_thread:
+                new_cls.clean_thread.stop()
+                new_cls.clean_thread.unsubscribe()
+                del new_cls.clean_thread
 
         @cherrypy.expose
         def index(self):
@@ -79,20 +88,20 @@ def setup_server():
         def delete(self):
             cherrypy.session.delete()
             sessions.expire()
-            return "done"
+            return 'done'
 
         @cherrypy.expose
         def delkey(self, key):
             del cherrypy.session[key]
-            return "OK"
+            return 'OK'
 
         @cherrypy.expose
-        def blah(self):
-            return self._cp_config['tools.sessions.storage_type']
+        def redir_target(self):
+            return self._cp_config['tools.sessions.storage_class'].__name__
 
         @cherrypy.expose
         def iredir(self):
-            raise cherrypy.InternalRedirect('/blah')
+            raise cherrypy.InternalRedirect('/redir_target')
 
         @cherrypy.expose
         @cherrypy.config(**{
@@ -105,7 +114,7 @@ def setup_server():
         @cherrypy.expose
         def regen(self):
             cherrypy.tools.sessions.regenerate()
-            return "logged in"
+            return 'logged in'
 
         @cherrypy.expose
         def length(self):
@@ -125,9 +134,6 @@ def setup_server():
     cherrypy.tree.mount(Root())
 
 
-from cherrypy.test import helper
-
-
 class SessionTest(helper.CPWebCase):
     setup_server = staticmethod(setup_server)
 
@@ -137,8 +143,9 @@ class SessionTest(helper.CPWebCase):
             if fname.startswith(sessions.FileSession.SESSION_PREFIX):
                 os.unlink(os.path.join(localDir, fname))
 
+    @pytest.mark.xfail(reason='#1534')
     def test_0_Session(self):
-        self.getPage('/setsessiontype/ram')
+        self.getPage('/set_session_cls/cherrypy.lib.sessions.RamSession')
         self.getPage('/clear')
 
         # Test that a normal request gets the same id in the cookies.
@@ -152,7 +159,7 @@ class SessionTest(helper.CPWebCase):
         self.getPage('/testStr')
         self.assertBody('1')
         cookie_parts = dict([p.strip().split('=')
-                             for p in self.cookies[0][1].split(";")])
+                             for p in self.cookies[0][1].split(';')])
         # Assert there is an 'expires' param
         self.assertEqual(set(cookie_parts.keys()),
                          set(['session_id', 'expires', 'Path']))
@@ -161,13 +168,14 @@ class SessionTest(helper.CPWebCase):
         self.getPage('/testStr', self.cookies)
         self.assertBody('3')
         self.getPage('/data', self.cookies)
-        self.assertBody("{'aha': 'foo', 'counter': 3}")
+        self.assertDictEqual(json_decode(self.body),
+                             {'counter': 3, 'aha': 'foo'})
         self.getPage('/length', self.cookies)
         self.assertBody('2')
         self.getPage('/delkey?key=counter', self.cookies)
         self.assertStatus(200)
 
-        self.getPage('/setsessiontype/file')
+        self.getPage('/set_session_cls/cherrypy.lib.sessions.FileSession')
         self.getPage('/testStr')
         self.assertBody('1')
         self.getPage('/testGen', self.cookies)
@@ -186,7 +194,7 @@ class SessionTest(helper.CPWebCase):
 
         # Test session __contains__
         self.getPage('/keyin?key=counter', self.cookies)
-        self.assertBody("True")
+        self.assertBody('True')
         cookieset1 = self.cookies
 
         # Make a new session and test __len__ again
@@ -196,9 +204,9 @@ class SessionTest(helper.CPWebCase):
 
         # Test session delete
         self.getPage('/delete', self.cookies)
-        self.assertBody("done")
+        self.assertBody('done')
         self.getPage('/delete', cookieset1)
-        self.assertBody("done")
+        self.assertBody('done')
         f = lambda: [
             x for x in os.listdir(localDir) if x.startswith('session-')]
         self.assertEqual(f(), [])
@@ -212,11 +220,12 @@ class SessionTest(helper.CPWebCase):
         self.assertEqual(f(), [])
 
     def test_1_Ram_Concurrency(self):
-        self.getPage('/setsessiontype/ram')
+        self.getPage('/set_session_cls/cherrypy.lib.sessions.RamSession')
         self._test_Concurrency()
 
+    @pytest.mark.xfail(reason='#1306')
     def test_2_File_Concurrency(self):
-        self.getPage('/setsessiontype/file')
+        self.getPage('/set_session_cls/cherrypy.lib.sessions.FileSession')
         self._test_Concurrency()
 
     def _test_Concurrency(self):
@@ -224,8 +233,8 @@ class SessionTest(helper.CPWebCase):
         request_count = 30
 
         # Get initial cookie
-        self.getPage("/")
-        self.assertBody("1")
+        self.getPage('/')
+        self.assertBody('1')
         cookies = self.cookies
 
         data_dict = {}
@@ -248,7 +257,7 @@ class SessionTest(helper.CPWebCase):
                 else:
                     data_dict[index] = max(data_dict[index], int(body))
                 # Uncomment the following line to prove threads overlap.
-##                sys.stdout.write("%d " % index)
+                # sys.stdout.write("%d " % index)
 
         # Start <request_count> requests from each of
         # <client_thread_count> concurrent clients
@@ -273,17 +282,19 @@ class SessionTest(helper.CPWebCase):
         # Start a new session
         self.getPage('/testStr')
         self.getPage('/iredir', self.cookies)
-        self.assertBody("file")
+        self.assertBody('FileSession')
 
+    @pytest.mark.xfail(reason='#1540')
     def test_4_File_deletion(self):
         # Start a new session
         self.getPage('/testStr')
         # Delete the session file manually and retry.
-        id = self.cookies[0][1].split(";", 1)[0].split("=", 1)[1]
-        path = os.path.join(localDir, "session-" + id)
+        id = self.cookies[0][1].split(';', 1)[0].split('=', 1)[1]
+        path = os.path.join(localDir, 'session-' + id)
         os.unlink(path)
         self.getPage('/testStr', self.cookies)
 
+    @pytest.mark.xfail(reason='#1557')
     def test_5_Error_paths(self):
         self.getPage('/unknown/page')
         self.assertErrorPage(404, "The path '/unknown/page' was not found.")
@@ -299,31 +310,31 @@ class SessionTest(helper.CPWebCase):
     def test_6_regenerate(self):
         self.getPage('/testStr')
         # grab the cookie ID
-        id1 = self.cookies[0][1].split(";", 1)[0].split("=", 1)[1]
+        id1 = self.cookies[0][1].split(';', 1)[0].split('=', 1)[1]
         self.getPage('/regen')
         self.assertBody('logged in')
-        id2 = self.cookies[0][1].split(";", 1)[0].split("=", 1)[1]
+        id2 = self.cookies[0][1].split(';', 1)[0].split('=', 1)[1]
         self.assertNotEqual(id1, id2)
 
         self.getPage('/testStr')
         # grab the cookie ID
-        id1 = self.cookies[0][1].split(";", 1)[0].split("=", 1)[1]
+        id1 = self.cookies[0][1].split(';', 1)[0].split('=', 1)[1]
         self.getPage('/testStr',
                      headers=[
                          ('Cookie',
                           'session_id=maliciousid; '
                           'expires=Sat, 27 Oct 2017 04:18:28 GMT; Path=/;')])
-        id2 = self.cookies[0][1].split(";", 1)[0].split("=", 1)[1]
+        id2 = self.cookies[0][1].split(';', 1)[0].split('=', 1)[1]
         self.assertNotEqual(id1, id2)
         self.assertNotEqual(id2, 'maliciousid')
 
     def test_7_session_cookies(self):
-        self.getPage('/setsessiontype/ram')
+        self.getPage('/set_session_cls/cherrypy.lib.sessions.RamSession')
         self.getPage('/clear')
         self.getPage('/session_cookie')
         # grab the cookie ID
         cookie_parts = dict([p.strip().split('=')
-                            for p in self.cookies[0][1].split(";")])
+                            for p in self.cookies[0][1].split(';')])
         # Assert there is no 'expires' param
         self.assertEqual(set(cookie_parts.keys()), set(['temp', 'Path']))
         id1 = cookie_parts['temp']
@@ -332,7 +343,7 @@ class SessionTest(helper.CPWebCase):
         # Send another request in the same "browser session".
         self.getPage('/session_cookie', self.cookies)
         cookie_parts = dict([p.strip().split('=')
-                            for p in self.cookies[0][1].split(";")])
+                            for p in self.cookies[0][1].split(';')])
         # Assert there is no 'expires' param
         self.assertEqual(set(cookie_parts.keys()), set(['temp', 'Path']))
         self.assertBody(id1)
@@ -342,7 +353,7 @@ class SessionTest(helper.CPWebCase):
         self.getPage('/session_cookie')
         # grab the cookie ID
         cookie_parts = dict([p.strip().split('=')
-                            for p in self.cookies[0][1].split(";")])
+                            for p in self.cookies[0][1].split(';')])
         # Assert there is no 'expires' param
         self.assertEqual(set(cookie_parts.keys()), set(['temp', 'Path']))
         # Assert a new id has been generated...
@@ -356,9 +367,9 @@ class SessionTest(helper.CPWebCase):
         cache = copykeys(sessions.RamSession.cache)
         if cache:
             if cache == [id2]:
-                self.fail("The second session did not time out.")
+                self.fail('The second session did not time out.')
             else:
-                self.fail("Unknown session id in cache: %r", cache)
+                self.fail('Unknown session id in cache: %r', cache)
 
     def test_8_Ram_Cleanup(self):
         def lock():
@@ -369,13 +380,16 @@ class SessionTest(helper.CPWebCase):
 
         t = threading.Thread(target=lock)
         t.start()
+        start = time.time()
+        while not sessions.RamSession.locks and time.time() - start < 5:
+            time.sleep(0.01)
+        assert len(sessions.RamSession.locks) == 1, 'Lock not acquired'
         s2 = sessions.RamSession()
         s2.clean_up()
-        self.assertEqual(len(sessions.RamSession.locks), 1, 'Clean up should not remove active lock')
+        assert len(sessions.RamSession.locks) == 1, 'Clean up should not remove active lock'
         t.join()
 
 
-import socket
 try:
     import memcache  # NOQA
 
@@ -401,13 +415,13 @@ except (ImportError, socket.error):
         setup_server = staticmethod(setup_server)
 
         def test(self):
-            return self.skip("memcached not reachable ")
+            return self.skip('memcached not reachable ')
 else:
     class MemcachedSessionTest(helper.CPWebCase):
         setup_server = staticmethod(setup_server)
 
         def test_0_Session(self):
-            self.getPage('/setsessiontype/memcached')
+            self.getPage('/set_session_cls/cherrypy.Sessions.MemcachedSession')
 
             self.getPage('/testStr')
             self.assertBody('1')
@@ -417,7 +431,7 @@ else:
             self.assertBody('3')
             self.getPage('/length', self.cookies)
             self.assertErrorPage(500)
-            self.assertInBody("NotImplementedError")
+            self.assertInBody('NotImplementedError')
             self.getPage('/delkey?key=counter', self.cookies)
             self.assertStatus(200)
 
@@ -428,28 +442,28 @@ else:
 
             # Test session __contains__
             self.getPage('/keyin?key=counter', self.cookies)
-            self.assertBody("True")
+            self.assertBody('True')
 
             # Test session delete
             self.getPage('/delete', self.cookies)
-            self.assertBody("done")
+            self.assertBody('done')
 
         def test_1_Concurrency(self):
             client_thread_count = 5
             request_count = 30
 
             # Get initial cookie
-            self.getPage("/")
-            self.assertBody("1")
+            self.getPage('/')
+            self.assertBody('1')
             cookies = self.cookies
 
             data_dict = {}
 
             def request(index):
                 for i in range(request_count):
-                    self.getPage("/", cookies)
+                    self.getPage('/', cookies)
                     # Uncomment the following line to prove threads overlap.
-##                    sys.stdout.write("%d " % index)
+                    # sys.stdout.write("%d " % index)
                 if not self.body.isdigit():
                     self.fail(self.body)
                 data_dict[index] = int(self.body)
@@ -474,7 +488,7 @@ else:
             # Start a new session
             self.getPage('/testStr')
             self.getPage('/iredir', self.cookies)
-            self.assertBody("memcached")
+            self.assertBody('memcached')
 
         def test_5_Error_paths(self):
             self.getPage('/unknown/page')
